// Temporary proxy for letting existing code find bundled NL include files
// without changing it

#ifndef GEOGRAM_NL_PRECONDITIONERS
#define GEOGRAM_NL_PRECONDITIONERS

#ifdef __cplusplus
extern "C" {
#endif

#include <geogram/third_party/OpenNL/nl_preconditioners.h>

#ifdef __cplusplus
}
#endif

#endif
