// Temporary proxy for letting existing code find bundled NL include files
// without changing it

#ifndef GEOGRAM_NL_ITERATIVE_SOLVERS
#define GEOGRAM_NL_ITERATIVE_SOLVERS

#ifdef __cplusplus
extern "C" {
#endif

#include <geogram/third_party/OpenNL/nl_iterative_solvers.h>

#ifdef __cplusplus
}
#endif

#endif
